"use strict";
exports.__esModule = true;
exports.ObjectRules = void 0;
var ObjectRules = /** @class */ (function () {
    function ObjectRules() {
        var _this = this;
        this.register = function () {
            _this.rules.set({ name: 'Get', group: 'Object' }, _this.get);
            _this.rules.set({ name: 'Set', group: 'Object' }, _this.set);
            _this.rules.set({ name: 'Update', group: 'Object' }, _this.update);
            _this.rules.set({ name: 'Delete', group: 'Object' }, _this["delete"]);
        };
        this.isArrayOfPaths = function (path) {
            if (!Array.isArray(path))
                return false;
            return path.every(function (p) { return typeof p === 'string'; });
        };
        this.baseGet = function (pathProps, data) {
            var property = pathProps.shift();
            if (!property || data[property] === undefined)
                return undefined;
            return pathProps.length === 0 ? (property === '*' ? data : data[property]) : _this.baseGet(pathProps, data[property]);
        };
        this.baseSet = function (pathProps, data, value, upsert) {
            if (upsert === void 0) { upsert = false; }
            var lastKeyIndex = pathProps.length - 1;
            for (var i = 0; i < lastKeyIndex; ++i) {
                var property = pathProps[i];
                if (!(property in data)) {
                    data[property] = {};
                }
                data = data[property];
            }
            data[pathProps[lastKeyIndex]] = value;
            return true;
        };
        this.baseUpdate = function (pathProps, data, value) {
            var property = pathProps.shift();
            if (!property || data[property] === undefined)
                return false;
            if (pathProps.length === 0) {
                data[property] = value;
                return true;
            }
            else {
                return _this.baseUpdate(pathProps, data[property], value);
            }
        };
        this.baseDelete = function (pathProps, data) {
            var property = pathProps.shift();
            if (!property || data[property] === undefined)
                return false;
            if (pathProps.length === 0) {
                delete data[property];
                return true;
            }
            else {
                return _this.baseDelete(pathProps, data[property]);
            }
        };
        this.set = function (path, value, data) {
            var pathProps = _this.getPath(path);
            return data && pathProps && value ? _this.baseSet(pathProps, data, value, true) : false;
        };
        this.get = function (path, defaultValue, data) {
            var pathProps = _this.getPath(path);
            // defaultValue = defaultValue || 'invalidPath';
            var result = data && pathProps ? _this.baseGet(pathProps, data) : undefined;
            return result === undefined ? defaultValue : result;
        };
        this.update = function (path, value, data) {
            var pathProps = _this.getPath(path);
            return data && pathProps && value ? _this.baseUpdate(pathProps, data, value) : false;
        };
        this["delete"] = function (path, data) {
            var pathProps = _this.getPath(path);
            return data && pathProps ? _this.baseDelete(pathProps, data) : false;
        };
        this.arrayEleNumRegex = /^\[([0-9]+)\]/;
        this.rules = new Map();
        this.register();
    }
    ObjectRules.prototype.getRules = function () {
        return this.rules;
    };
    ObjectRules.prototype.getPath = function (path) {
        var pathProps = path;
        if (typeof path === 'string')
            pathProps = path.split('.');
        return this.isArrayOfPaths(pathProps) && pathProps.length > 0 ? pathProps : undefined;
    };
    return ObjectRules;
}());
exports.ObjectRules = ObjectRules;
//# sourceMappingURL=object.js.map